from module.util.console import Console
from module.datasource import Datasource

Console.log('==================================================================================================')
Console.log('===                                       New run                                              ===')
Console.log('==================================================================================================')
#test = Datasource('test.json')

#test.drop()
#test.create()

#arr = {'addressID':100,'fdfd':1,'code':'something'}

#result = test.insert(arr)
#Console.log(test.id)
#Console.log(result)
#Console.log(test.error)
#Console.log(test.data)
#test.insert()
#test.insert()
#test.insert()


#test.where([{'ID',2}])
#test.field(['ID','addressID','stuff']).select()
#print(test.data)




# static class, no self here
class Shell:
    @staticmethod
    def run(arg):
        import subprocess

        result = subprocess.Popen(arg,
                        shell=True,
                        stdout=subprocess.PIPE, 
                        stderr=subprocess.PIPE,
                        universal_newlines=True) 
        stdout, stderr = result.communicate()

        if stderr:
            print(stderr)

        if (result.returncode != 0):
            print(f'Shell command failed "{arg}"')
            #exit()

        return stdout, stderr


from module.base import base
import json

class nodetool(base):

    def __init__(self, dev=0):
        super().__init__()
        self.cmd = {
            "status": "nodetool status",
            "gcstats": "nodetool gcstats",
            "info": "nodetool info",
            "ring": "nodetool ring",
            "tablestats": "nodetool tablestats -F json",
            "profileload": "nodetool profileload",
            "histograms": "nodetool tablehistograms"
        }
        
    def status(self):
        out, err = self.runShell(self.cmd["status"])
        out = self.processShellResult(out)

        status = {"UN": "Up","UL": "Up","UJ": "Up","UM": "Up","DN": "Down","DL": "Down","DJ": "Down","DM": "Down"}
        state = {"UN": "Normal","UL": "Leaving","UJ": "Joining","UM": "Moving","DN": "Normal","DL": "Leaving","DJ": "Joining","DM": "Moving"}

        result = {}
        result["datacenter"] = out[1][2]
        result["host"] = []

        for key, line in out.items(): 
            if (key >= 6):
                temp = {}
                #result["host"][ line[2] ] = {}
                temp["status"] = status.get(line[1], "Unknown status")
                temp["state"] = state.get(line[1], "Unknown state")
                temp["load"] = line[5]
                temp["owns"] = line[6]
                temp["hostID"] = line[7]
                temp["rack"] = line[8]
                result["host"].append(temp)


        return json.dumps(result), err
    
    def info(self):
        out, err = self.runShell(self.cmd["info"])
        out = self.processShellResult(out, seperator=":")

        result = {}
        for line in out:
            result[ out[line][1] ] = {}
            result[ out[line][1] ] = out[line][2]

        result["load"] = self._convertSize(result["load"])
        result["key cache"] = self._info_cache(result["key cache"])
        result["row cache"] = self._info_cache(result["row cache"])
        result["counter cache"] = self._info_cache(result["counter cache"])
        
        return json.dumps(result), err

    def _info_cache(self, cacheStr):
        counter = 0
        temp, result = {}, {}

        for line in cacheStr.split(","):
            counter = counter + 1
            temp[counter] = line.strip()

        result["entries"] = temp[1].replace('entries', '').strip()
        result["size"] = self._convertSize(temp[2].replace('size', '').strip())
        result["capacity"] = self._convertSize(temp[3].replace('capacity', '').strip())
        result["hits"] = temp[4].replace('hits', '').strip()
        result["requests"] = temp[5].replace('requests', '').strip()
        result["hit_rate"] = self._convertSize(temp[6].replace('recent hit rate', '').strip())
        result["save_period"] = temp[7].replace('save period in seconds', '').strip()

        return result

    def profileload(self):
        out, err = self.runShell(self.cmd["profileload"])
        print(out)
        out = self.processShellResult(out, seperator=":")

        print(out)
        return json.dumps(out), err
    
    def histograms(self):
        out, err = self.runShell(self.cmd["histograms"])
        print(out)
        out = self.processShellResult(out, seperator=":")

        print(out)
        return json.dumps(out), err

    def tablestats(self):
        out, err = self.runShell(self.cmd["tablestats"])
        return out, err


from module.util.console import Console
import os
from typing import Any

class File:

    _filePath = ''
    result: Any = None

    def __init__(self, filePath):
        # figure out what type of file this is

        filename, file_extension = os.path.splitext(filePath)

        do = f"_ext_{file_extension[1:]}"
  
        if hasattr(self, do) and callable(func := getattr(self, do)):
            self._filePath = self._buildFilePath(filePath)
            self.result = func()



    def _ext_json(self):
        Console.info(f'Loading json data from "{self._filePath}"')
        import json

        result = {}
        try:
            data = open(self._filePath)
            result = json.load(data)
            
            Console.ok('File opened and parsed')
        except:
            Console.error('Error when opening file')
       
        return result

    def _buildFilePath(self, filePath):
        # we take the file path passed in and do some checks to make sure it will work

        filePath = filePath.replace('/', os.path.sep)
        filePath = filePath.replace('\\', os.path.sep)

        return filePath


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ivyorm',
 'ivyorm.module',
 'ivyorm.module.connection',
 'ivyorm.module.connectors',
 'ivyorm.module.interface',
 'ivyorm.module.util']

package_data = \
{'': ['*'],
 'ivyorm': ['ivypy.egg-info/*', 'model/*'],
 'ivyorm.module': ['connectors/keys/*']}

setup_kwargs = {
    'name': 'ivyorm',
    'version': '0.1.6',
    'description': 'An ORM for Python',
    'long_description': '# article i followed for creating a package\nhttps://mathspp.com/blog/how-to-create-a-python-package-in-2022\n\n# first, we need pipx in order to install poetry\npython -m pip install --user pipx\n\n# make sure the right python scripts folder is in the PATH env varaible on windows\n\npipx install poetry\n\n#update the PATH env\npipx ensurepath\n\n# reload the shell\n\n# for a new project, run \'poetry new .\' in the new empty directory\n# for an existing project, type the below in the the directory and follow the prompts\npoetry init\n\n# now run install\npoetry install\n\n# do the git stuff if a new project that\'s not on git yet\ngit init\ngit add *\ngit commit -m "First commit"\ngit branch -M main\ngit remote add origin https://github.com/JamesRandell/ivypy.git\ngit push -u origin main\n\n# more poetry stuff \npoetry add -D pre-commit\n\npoetry config repositories.testpypi https://test.pypi.org/legacy/\n\n\n#api token\n\n# test account\npoetry config http-basic.testpypi __token__ test-token-here\n\npoetry build\npoetry publish -r testpypi\n\n# live account\npoetry config pypi-token.pypi live-token-here\n\n\npoetry publish --build\n',
    'author': 'James Randell',
    'author_email': 'jamesrandell@me.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



from module.shell import Shell

class Conn(Shell):



    def __init__(self, config):
        self.shell_string = '' # this is what our app will use to prefix any commands we send to the database on the shell 

        self.config = config # not used right now as i'm just dealing with local docker

        container_id = self.get_container_id()

        
        
        self.build_shell_string(container_id)


    def build_shell_string(self, container_id):
        self.shell_string = f'docker exec {container_id}'


        

    def get_container_id(self):

        o, r = Shell.run(f'docker ps -f name="cassandra" --format "{{{{.ID}}}}"')

        return o.rstrip("\n")